
-- original impliedpermissions function
CREATE FUNCTION dbo.ImpliedPermissions1(@class nvarchar(64), @permission_name nvarchar(64))
RETURNS TABLE
AS RETURN
SELECT a.*, b.height, b.RANK
FROM sys.fn_builtin_permissions('') as a
CROSS APPLY master.dbo.ImplyingPermissions(a.class_desc, a.permission_name) as b
where b.CLASS = @class and b.permname = @permission_name
GO

select * from dbo.ImpliedPermissions('SERVER', 'CONTROL SERVER');
select * from dbo.ImpliedPermissions('SERVER', 'ALTER ANY LOGIN');
select * from dbo.ImpliedPermissions('DATABASE', 'CONTROL') -- 167 permissions
order by class_desc, permission_name, height, RANK;

-- ALTER ANY DATABASE doesn't give me CONTROL DATABASE in any of them
select * from dbo.ImpliedPermissions('SERVER', 'ALTER ANY DATABASE') -- 66 permissions
order by class_desc, permission_name, height, RANK;
-- let's check
select * from dbo.ImplyingPermissions('DATABASE', 'CONTROL'); -- that's correct

-- in fact, we could see which ones you get from each, that you don't get from the other
-- 104 permissions out of 167 from CONTROL DATABASE aren't in ALTER ANY DATABASE
select class_desc, permission_name from dbo.ImpliedPermissions('DATABASE', 'CONTROL')
EXCEPT 
select class_desc, permission_name from dbo.ImpliedPermissions('SERVER', 'ALTER ANY DATABASE')

-- 3 permissions out of 66 from ALTER ANY DATABASE aren't in CONTROL DATABASE
select class_desc, permission_name from dbo.ImpliedPermissions('SERVER', 'ALTER ANY DATABASE')
EXCEPT 
select class_desc, permission_name from dbo.ImpliedPermissions('DATABASE', 'CONTROL')

-- that should leave 63 that are implied by either permission
-- this does take 2 minutes or so to run however!
select class_desc, permission_name from dbo.ImpliedPermissions('SERVER', 'ALTER ANY DATABASE')
INTERSECT
select class_desc, permission_name from dbo.ImpliedPermissions('DATABASE', 'CONTROL')

-- this one's even longer - 5:24!!
-- since the table of implied permissions only consists of 900-something rows, we could persist it and index it
select class_desc, permission_name from dbo.ImpliedPermissions('SERVER', 'ALTER ANY DATABASE') a 
where exists 
( select class_desc, permission_name from dbo.ImpliedPermissions('DATABASE', 'CONTROL') b
  where a.class_desc = b.class_desc and a.permission_name = b.permission_name
)

-- make table of all implied permissions
-- 995 rows affected in 40 seconds, I think I'm running out of memory
SELECT * INTO #ImpliedPermissions
FROM sys.fn_builtin_permissions('') as a
CROSS APPLY master.dbo.ImplyingPermissions(a.class_desc, a.permission_name) as b
GO

CREATE DATABASE Secutil;
GO

USE Secutil
GO

drop function dbo.ImpliedPermissons
drop function dbo.CalcPermissions;
drop table dbo.ImpliedPermissions

SELECT * INTO Secutil.dbo.ImpliedPermissionsList
FROM sys.fn_builtin_permissions('') as a
CROSS APPLY master.dbo.ImplyingPermissions(a.class_desc, a.permission_name) as b
GO

CREATE FUNCTION dbo.ImpliedPermissions(@class nvarchar(64), @permission_name nvarchar(64))
RETURNS TABLE
AS RETURN
SELECT class_desc, permission_name, type, covering_permission_name, parent_class_desc, parent_covering_permission_name, height, RANK 
FROM Secutil.dbo.ImpliedPermissionsList 
WHERE CLASS = @class AND permname = @permission_name
GO

select * from dbo.ImpliedPermissions('SERVER', 'CONTROL SERVER'); -- 219 rows
select distinct class_desc, permission_name from dbo.ImpliedPermissons('SERVER', 'CONTROL SERVER'); -- 219 rows
select * from dbo.ImpliedPermissions('DATABASE', 'CONTROL'); -- 167
select distinct class_desc, permission_name from dbo.ImpliedPermissons('DATABASE', 'CONTROL'); -- 167
select * from dbo.ImpliedPermissions('SERVER', 'ALTER ANY DATABASE'); -- 66
select distinct class_desc, permission_name from dbo.ImpliedPermissons('SERVER', 'ALTER ANY DATABASE'); -- 66

-- 63 rows, subsecond
select class_desc, permission_name from Secutil.dbo.ImpliedPermisisonsList('DATABASE', 'CONTROL')
INTERSECT
select class_desc, permission_name from Secutil.dbo.ImpliedPermissionsList('SERVER', 'ALTER ANY DATABASE')
go

-- a DENY calculator
-- the first two parameters are the one you are granted
-- the second two are the one you are denied
CREATE FUNCTION dbo.CalcPermissions(@class nvarchar(64), @permission_name nvarchar(64), @d_class nvarchar(64), @d_permission_name nvarchar(64))
RETURNS TABLE
AS RETURN
-- no height and rank here, I'm don't care how you get or lose the permissions
SELECT class_desc, permission_name, type, covering_permission_name, parent_class_desc, parent_covering_permission_name
FROM Secutil.dbo.ImpliedPermissionsList 
WHERE CLASS = @class AND permname = @permission_name
EXCEPT
SELECT class_desc, permission_name, type, covering_permission_name, parent_class_desc, parent_covering_permission_name
FROM Secutil.dbo.ImpliedPermissionsList
WHERE CLASS = @d_class AND permname = @d_permission_name
GO

-- GRANTED CONTROL DATABASE on a specific DATABASE, DENIED ALTER_ANY_DATABASE
-- 104 rows
select class_desc, permission_name from dbo.CalcPermissions('DATABASE', 'CONTROL', 'SERVER', 'ALTER ANY DATABASE')

-- 217 rows
-- this doesn't account for the fact that DENYing IMPERSONATE ANY LOGIN will also deny IMPERSONATE USER
select class_desc, permission_name from dbo.CalcPermissions('SERVER', 'CONTROL SERVER', 'SERVER', 'IMPERSONATE ANY LOGIN')